# Programmer SWD D11C

We start out with one that is already pre-programmed. Then we take another one, that is blank.

## Getting EDBG

You will need EDBG command line program. You can find it here.
- https://taradov.com/bin/edbg/

First, set up the permissions, so that you can run it from terminal. Open download location, right click on the downloaded file and select "Properties". There go to "Permissions" tab and make sure the "Allow executing file..." checkbox is checked.

If you do not see such option, you will need to use command line to give permission to run the edbg program. Go to location with edbg and then use that location with command below, where the usr/bin part is replaced with the actual location on your device and the edbg version matches the one you have downloaded (for example edbg-linux-r61)

```
sudo chmod a+x /usr/local/bin/edbg-linux-r61
```

One way to run it is from the current directory with the "./edbg..." syntax. If you want to use it from whatever location on the system, which is the case when you have multiple projects that reside in different directories, you have to move the "edbg..." file to "/usr/local/bin" folder on Linux and Mac systems. To do that, in Terminal, make sure that you are in the directory where the "edbg..." file resides. Use the following command.

```
sudo mv edbg-linux-r61 /usr/local/bin/edbg
```

The above example is for Linux, but on Mac it should be the same, except the linux is going to be "mac". What will happen is that Terminal will ask for the admin password (which you will happily provide) and it will move and rename the file "edbg-linux-r61" to "/usr/local/bin" folder with the name "edbg". From now on you do not need to use the full name for example "edbg-linux-r61", it has been now renamed to "edbg". ** Make sure the command line commands match the corresponding file names and paths you have on your system. **

To verify that you can use edbg, type ´edbg --help´ and hit enter. You should see something like the following.

```
CMSIS-DAP SWD programmer. Built Feb 13 2024 00:42:17.

Usage: edbg [options]
Options:
...
```

The above will confirm that the software is installed successfuly.

## Flashing Programmer Firmware

For the programmer to become a funcional SWD programmer, it needs firmware. This is the so called Free DAP firmware. The file name you are looking for is ´free_dap_d11c_mini.bin´ and it is available for [download here](http://academy.cba.mit.edu/classes/embedded_programming/SWD/free_dap_d11c_mini.bin) and also in this repository. 

Then, in Terminal, navigate to directory where the above-metioned file is. Next, make sure that the existing and working programmer is connected and ´edbg´ can see it. To verify that, type the following.

```
edbg --list
```

You should see an output like the one below.

```
Attached debuggers:
  0: 4344C80D - Alex Taradov Generic CMSIS-DAP Adapter (1)
```

If you do not see it, double check the connections (computer - USB - programmer).

Then, connect the 2x2 connector programming cable from the connector on the edge of the programmer board to the connector closer to the chip on the other (not yet programmed) programmer board. The boards should be aligned in the same way, meaning that the USB plugs should point in the same direction.

![ SWD D11C programmer](../images/d11c_programmer.jpg)

Make sure that the other board is also connected to a USB port. It needs voltage to be able to communicate with the programmer.

To flash the firmware, use the following command. Make sure that the ´free_dap_d11c_mini.bin´ file is in the directory where you are currently. 

```
edbg -ebpv -t samd11 -f free_dap_d11c_mini.bin
```

Hint: if you see multiple programmers attached, you can use ´--serial´ flag to specify the serial number of a specific programmer that you want to use. Like below.

```
edbg -ebpv -t samd11 -f free_dap_d11c_mini.bin --serial 4344C80D
```

Anyhow, in the case of success, you should see the following output.

```
Debugger: Alex Taradov Generic CMSIS-DAP Adapter 4344C80D v0.1 (S, 1)
Using CMSIS-DAP v1
Clock frequency: 16.0 MHz
Target: SAM D11C14A (Rev B)
Erasing... done.
Programming... done.
Verification... done.
```

## Using the Programmer

This programmer can be used to program the ´SAMD´ family chips, such as the same chip that is used in the programmer, the ´D11C´.

To test the programmer, you can use the ´Hello D11C´ or the 'Hello D11C USB-C' board.

First, you need to flash the bootloader, using the SWD connection. Once it is done, you will be able to use the USB connection to program the Hello board using Arduino.

To flash the bootloader, you need to look for the file named ´sam_ba_SAMD11C14A.bin´. You can [download it here](https://github.com/qbolsee/ArduinoCore-fab-sam/blob/master/bootloaders/zero/binaries/sam_ba_SAMD11C14A.bin?raw=true) and it should be available in this repository.

Next, in Terminal, you should navigate to the file location. Then, make sure your programmer and hello board are connected correctly. The programmer should be plugged in via USB connection to the computer. An SWD cable should be connected to the connector on the edge of the programmer and to the 2x2 connector closer to the chip on the Hello board. The boards hould be aligned so that USB connectors face the same direction. Then, use the following command to flash the bootloader.

![D11C Hello Board](../images/hello_d11c.jpg)

```
edbg -ebpv -t samd11 -f sam_ba_SAMD11C14A.bin
```

Again, if you have multiple programmers attached, you may use the ´--serial´ specifier.

You should see the following output on success.

```
Debugger: Alex Taradov Generic CMSIS-DAP Adapter 4344C80D v0.1 (S, 1)
Using CMSIS-DAP v1
Clock frequency: 16.0 MHz
Target: SAM D11C14A (Rev B)
Erasing... done.
Programming.... done.
Verification.... done.
```

Now, you can program the hello board using Arduino. You can disconnect the programmer now if you want. 

> **Important**: Use the [1.8.X](https://www.arduino.cc/en/software#legacy-ide-18x) version of the Arduino IDE (the older one). At the moment of writing this the recommended version is 1.8.19.

In Arduino, you will need to install additional board definitions. Refer to [this tutorial](https://gitlab.cba.mit.edu/quentinbolsee/Arduino-SAM-tutorial/) in order to get them installed.

As a result you should see the ´Fab SAM core for Arduino´ listed under "Tools > Boards" in Arduino IDE. You should also see a serial port available under "Tools > Port".

1. Select board: "Tools > Board > Fab SAM core for Arduino > Generic D11C14A"
2. Select port: "Tools > Port > /dev/ttyACM0"

Now, try the example code below or whatever code you would like to upload. 

```
void setup() {
  pinMode(5, OUTPUT);
}

void loop() {
  digitalWrite(5, HIGH);
  delay(1000);
  digitalWrite(5, LOW);
  delay(1000);
}
```

Go to "Sketch > Upload" to compile and upload your sketch. You should see the board LED blinking on success as well as the following output in the Arduino console.

```
Write 10116 bytes to flash (159 pages)
[==============================] 100% (159/159 pages)
done in 2.038 seconds

Verify 10116 bytes of flash
[==============================] 100% (159/159 pages)
Verify successful
done in 0.075 seconds
CPU reset.
```

## Conclusions

At this point you can tap yourself on the shoulder and congratulate yourself as you have a working SWD programmer that can program other SAM boards! 

